package com.bpcl.lpg.core.helper;

import org.slf4j.LoggerFactory;
import com.bpcl.core.util.BPCLDateUtils;
import com.bpcl.facades.payment.refund.RefundMsdData;
import com.bpcl.lpg.core.model.LpgPaymentInfoModel;
import org.apache.commons.lang.StringUtils;
import java.text.DateFormat;
import java.util.Date;
import java.util.TimeZone;
import java.text.SimpleDateFormat;
import com.bpcl.facades.payment.refill.RefillBookingData;
import com.bpcl.lpg.core.pojo.LpgPaymentResponseData;
import java.text.ParseException;
import com.bpcl.lpg.facades.data.LpgPaymentRequestData;
import com.bpcl.lpg.facades.data.LpgPaymentQueryData;
import com.bpcl.lpg.core.pojo.BillDeskResponseMsgData;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import de.hybris.platform.servicelayer.keygenerator.KeyGenerator;
import com.bpcl.lpg.core.encryption.LpgEncryptionService;
import org.slf4j.Logger;

public class LpgRefillHelper
{
    private static final Logger LOG;
    private static final String CONFIRM_REFILL_PAYMENT = "18";
    private static final String CONFIRM_REFILL_PROP_NAME = "confirmRefillPaymentUrl";
    private LpgEncryptionService lpgEncryptionService;
    private KeyGenerator paymentTransactionIdGenerator;
    private LPGCredentialsService lpgCredentialsService;
    
    public LpgPaymentRequestData prepareLpgPaymentRequestData(final BillDeskResponseMsgData responseMsgData, final LpgPaymentQueryData queryData) throws ParseException {
        final LpgPaymentRequestData lpgPaymentRequestData = this.prepareRefillData(responseMsgData, queryData);
        lpgPaymentRequestData.setAPI_Req_No("18");
        lpgPaymentRequestData.setEndPoint("confirmRefillPaymentUrl");
        lpgPaymentRequestData.setHashValue(this.getHashKeyValueRequest(lpgPaymentRequestData));
        return lpgPaymentRequestData;
    }
    
    public RefillBookingData prepareRefillBookingData(final LpgPaymentResponseData responseData) {
        final RefillBookingData refillBookingData = new RefillBookingData();
        refillBookingData.setRefillStatusCode(responseData.getStatus_Code());
        refillBookingData.setBpclUniqueID(responseData.getBpclUniqueID());
        refillBookingData.setDeliveryCompletionDate(responseData.getDeliveryCompletionDate());
        refillBookingData.setDistCode(responseData.getDistCode());
        refillBookingData.setOrderAmount(responseData.getOrderAmount());
        refillBookingData.setPaymentReceivedOn(responseData.getPaymentReceivedOn());
        refillBookingData.setProductCode(responseData.getProductCode());
        refillBookingData.setProductName(responseData.getProductName());
        refillBookingData.setProductQty(responseData.getProductQty());
        refillBookingData.setRefillBookDate(responseData.getBookDate());
        refillBookingData.setRefillOrderNo(responseData.getOrderNo());
        return refillBookingData;
    }
    
    private String getFormattedCurrentDate() {
        final DateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("IST"));
        return simpleDateFormat.format(new Date());
    }
    
    private String getHashKeyValueRequest(final LpgPaymentRequestData lpgPaymentRequestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = lpgPaymentRequestData.getEndPoint();
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        lpgPaymentRequestData.setPartner_Code(partnerCode);
        hashMessage.append(partnerCode);
        hashMessage.append(lpgPaymentRequestData.getPartner_Trans_ID());
        hashMessage.append(lpgPaymentRequestData.getDist_Ref_No());
        hashMessage.append(lpgPaymentRequestData.getLPG_ID());
        hashMessage.append(lpgPaymentRequestData.getDist_Code());
        hashMessage.append(lpgPaymentRequestData.getProduct_Code());
        hashMessage.append(lpgPaymentRequestData.getProduct_Qty());
        hashMessage.append(lpgPaymentRequestData.getPayment_Method());
        hashMessage.append(lpgPaymentRequestData.getPayment_Date());
        hashMessage.append(lpgPaymentRequestData.getOrder_Amount());
        hashMessage.append(lpgPaymentRequestData.getCollected_Amount());
        hashMessage.append(lpgPaymentRequestData.getSettlement_Amount());
        hashMessage.append(lpgPaymentRequestData.getAPI_Req_No());
        hashMessage.append(lpgPaymentRequestData.getAddl1());
        hashMessage.append(lpgPaymentRequestData.getAddl2());
        hashMessage.append(lpgPaymentRequestData.getAddl3());
        hashMessage.append(secret);
        final String encryptedMsg = this.getLpgEncryptionService().encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    public LpgPaymentRequestData prepareRefillData(final BillDeskResponseMsgData responseMsgData, final LpgPaymentQueryData queryData) throws ParseException {
        final LpgPaymentRequestData data = new LpgPaymentRequestData();
        data.setPartner_Trans_ID(responseMsgData.getCustomerID());
        data.setDist_Ref_No(responseMsgData.getCustomerID());
        data.setLPG_ID(responseMsgData.getAdditionalInfo1());
        data.setDist_Code(responseMsgData.getAdditionalInfo2());
        data.setProduct_Code(responseMsgData.getAdditionalInfo3());
        data.setProduct_Qty(responseMsgData.getAdditionalInfo4());
        data.setPayment_Method(responseMsgData.getAdditionalInfo5());
        this.populateQueryDate(responseMsgData, queryData, data);
        final SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
        final Date billdesk_Date = dateFormat.parse(responseMsgData.getTxnDate());
        final SimpleDateFormat lpg_dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        final String payment_Date = lpg_dateFormat.format(billdesk_Date);
        data.setPayment_Date(payment_Date);
        this.setAmount(responseMsgData, data);
        data.setAddl1("");
        data.setAddl2("");
        data.setAddl3("");
        return data;
    }
    
    private void populateQueryDate(final BillDeskResponseMsgData responseMsgData, final LpgPaymentQueryData queryData, final LpgPaymentRequestData data) {
        if (queryData != null) {
            data.setLPG_ID(queryData.getLpgID());
            data.setDist_Code(queryData.getDistCode());
            data.setDist_Ref_No(responseMsgData.getCustomerID());
            data.setPayment_Method(queryData.getPaymentMethod());
            data.setProduct_Code(queryData.getProductCode());
            data.setProduct_Qty(queryData.getProductQuantity());
        }
    }
    
    private void setAmount(final BillDeskResponseMsgData responseMsgData, final LpgPaymentRequestData data) {
        final String txnAmount = responseMsgData.getTxnAmount();
        if (StringUtils.isNotBlank(txnAmount)) {
            final String amountInString = Double.valueOf(txnAmount).toString();
            data.setOrder_Amount(amountInString);
            data.setCollected_Amount(amountInString);
            data.setSettlement_Amount(amountInString);
        }
    }
    
    public RefundMsdData prepareRefundData(final LpgPaymentInfoModel paymentInfo) {
        final RefundMsdData refundMsdData = new RefundMsdData();
        refundMsdData.setBilldeskTxnID(paymentInfo.getBilldeskTxnReferenceID());
        refundMsdData.setRefundRefNo((String)this.getPaymentTransactionIdGenerator().generate());
        refundMsdData.setTxnAmount(paymentInfo.getTxnAmount().toString());
        refundMsdData.setTxnDate(BPCLDateUtils.dateToString(paymentInfo.getTxnDate(), "dd-MM-yyyy HH:mm:ss"));
        refundMsdData.setTransactionID(paymentInfo.getTransactionID());
        return refundMsdData;
    }
    
    public LpgEncryptionService getLpgEncryptionService() {
        return this.lpgEncryptionService;
    }
    
    public void setLpgEncryptionService(final LpgEncryptionService lpgEncryptionService) {
        this.lpgEncryptionService = lpgEncryptionService;
    }
    
    public KeyGenerator getPaymentTransactionIdGenerator() {
        return this.paymentTransactionIdGenerator;
    }
    
    public void setPaymentTransactionIdGenerator(final KeyGenerator paymentTransactionIdGenerator) {
        this.paymentTransactionIdGenerator = paymentTransactionIdGenerator;
    }
    
    public void setLpgCredentialsService(final LPGCredentialsService lpgCredentialsService) {
        this.lpgCredentialsService = lpgCredentialsService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgRefillHelper.class);
    }
}
